@@ -9,7 +9,7 @@ from rasterio import features
 from shapely.geometry import Polygon, shape
 from shapely.ops import unary_union
 from gisutils import shp2df, df2shp, get_proj_str
-from .gis import crs, read_polygon_feature, \
+from .gis import CRS, read_polygon_feature, \
     build_rtree_index, intersect
 from .units import convert_length_units
 
@@ -40,7 +40,7 @@ class Grid:
 
         # coordinate projection stuff
         self.model_units = model_units
-        self.crs = crs(epsg=epsg, proj_str=proj_str, prjfile=prjfile)
+        self.crs = CRS(epsg=epsg, proj_str=proj_str, prjfile=prjfile)
         if crs_units is not None:
             self.crs._length_units = crs_units
 
@@ -255,7 +255,7 @@ class StructuredGrid(Grid):
         """Check if cells are uniform by comparing their areas."""
         if self._uniform is None:
             areas = [g.area for g in self.df.geometry]
-            self._uniform = np.allclose(areas, np.mean(areas))
+            self._uniform = np.allclose(areas, np.mean(areas), rtol=0.01)
         return self._uniform
 
     @property
@@ -277,65 +277,31 @@ class StructuredGrid(Grid):
         SFR will be simulated (isfr) to a polygon (if multiple
         polygons area created, the largest one by area is retained).
         """
-        # vectorize the raster
-        shapes = features.shapes(self.isfr, transform=self.transform)
-        # convert the shapes corresponding to raster values of 1 to shapely objects
-        shapes = [shape(s[0]) for s in list(shapes) if s[1] == 1]
-        # get the shape with the largest area
-        areas = [s.area for s in shapes]
-        self._active_area = shapes[np.argmax(areas)]
+        if self.transform is not None:
+            # vectorize the raster
+            shapes = features.shapes(self.isfr, transform=self.transform)
+            # convert the shapes corresponding to raster values of 1 to shapely objects
+            shapes = list(shapes)
+            shapes = [shape(s[0]) for s in list(shapes) if s[1] == 1]
+            # get the shape with the largest area
+            areas = [s.area for s in shapes]
+            self._active_area = shapes[np.argmax(areas)]
+        else:
+            self._active_area = unary_union(self.df.loc[self.df.isfr == 1, 'geometry'])
 
     @classmethod
     def from_json(cls, jsonfile, active_area=None, isfr=None,
                   epsg=None, proj_str=None, prjfile=None):
-        from .utils import load_sr
-        sr = load_sr(jsonfile)
-        return cls.from_sr(sr, active_area=active_area, isfr=isfr,
-                           epsg=epsg, proj_str=proj_str, prjfile=prjfile)
+        from sfrmaker.fileio import load_modelgrid
+        grid = load_modelgrid(jsonfile)
+        return cls.from_modelgrid(grid, active_area=active_area, isfr=isfr,
+                                  epsg=epsg, proj_str=proj_str, prjfile=prjfile)
 
     @classmethod
     def from_sr(cls, sr=None, active_area=None, isfr=None,
                 epsg=None, proj_str=None, prjfile=None):
-        """Create StructureGrid class instance from a
-        flopy SpatialReference instance."""
-        vertices = sr.vertices
-        polygons = [Polygon(v) for v in sr.vertices]
-        df = pd.DataFrame({'node': range(0, len(vertices)),
-                           'i': sorted(list(range(sr.nrow)) * sr.ncol),
-                           'j': list(range(sr.ncol)) * sr.nrow,
-                           'geometry': polygons
-                           }, columns=['node', 'i', 'j', 'geometry'])
-        if epsg is None:
-            epsg = sr.epsg
-        if proj_str is None:
-            proj_str = sr.proj4_str
-        if prjfile is not None:
-            proj_str = get_proj_str(prjfile)
-        if isfr is not None:
-            # if a 3D array is supplied for isfr, convert to 2D
-            # (retain all i, j locations with at least one active layer;
-            # assuming that top of each highest-active cell represents the land surface)
-            if len(isfr.shape) == 3:
-                isfr = np.any(isfr == 1, axis=0).astype(int)
-                df['isfr'] = isfr.ravel()
-            elif isfr.shape == (sr.nrow, sr.ncol):
-                df['isfr'] = isfr.ravel()
-            else:
-                assert isfr.size == len(df), \
-                    "isfr must be of shape (nlay, nrow, ncol), (nrow, ncol) or (nrow * ncol,)"
-                df['isfr'] = isfr
-        uniform = False
-        dx, dy = None, None
-        if len(set(sr.delc)) == 1 and len(set(sr.delr)) == 1:
-            dx = sr.delr[0] * sr.length_multiplier
-            dy = sr.delc[0] * sr.length_multiplier
-            uniform = True
-        return cls.from_dataframe(df, uniform=uniform,
-                                  xul=sr.xul, yul=sr.yul, dx=dx, dy=dy,
-                                  rotation=sr.rotation,
-                                  model_units=sr.model_length_units,
-                                  bounds=sr.bounds, active_area=active_area,
-                                  epsg=epsg, proj_str=proj_str, prjfile=prjfile)
+        raise AttributeError("the from_sr method is depricated. Please use from_modelgrid,"
+                             "as demonstrated in the examples.")
 
     @classmethod
     def from_modelgrid(cls, mg=None, active_area=None, isfr=None,
@@ -410,7 +376,7 @@ class StructuredGrid(Grid):
                                   epsg=epsg, proj_str=proj_str, prjfile=prjfile)
 
     @classmethod
-    def from_dataframe(cls, df=None, uniform=False,
+    def from_dataframe(cls, df=None, uniform=None,
                        kcol='k', icol='i', jcol='j',
                        isfr_col='isfr',
                        geometry_column='geometry',
