@@ -8,13 +8,13 @@ import flopy
 from gisutils import shp2df, df2shp, project
 import sfrmaker
 from sfrmaker.routing import pick_toids, find_path, make_graph, renumber_segments
-from .checks import routing_is_circular, is_to_one
-from .gis import crs, read_polygon_feature, get_bbox
-from .grid import StructuredGrid
-from .nhdplus_utils import load_nhdplus_v2, get_prj_file
-from .sfrdata import SFRData
-from .units import convert_length_units, get_length_units
-from .utils import (width_from_arbolate_sum, arbolate_sum)
+from sfrmaker.checks import routing_is_circular, is_to_one
+from sfrmaker.gis import CRS, read_polygon_feature, get_bbox
+from sfrmaker.grid import StructuredGrid
+from sfrmaker.nhdplus_utils import load_nhdplus_v2, get_prj_file
+from sfrmaker.sfrdata import SFRData
+from sfrmaker.units import convert_length_units, get_length_units
+from sfrmaker.utils import (width_from_arbolate_sum, arbolate_sum)
 from sfrmaker.reaches import consolidate_reach_conductances, interpolate_to_reaches, setup_reach_data
 
 
@@ -57,12 +57,18 @@ class Lines:
         self.df = df
         self.attr_length_units = attr_length_units
         self.attr_height_units = attr_height_units
-        self.crs = crs(epsg=epsg, proj_str=proj_str, prjfile=prjfile)
+        self.crs = CRS(epsg=epsg, proj_str=proj_str, prjfile=prjfile)
         self._geometry_length_units = None
 
         self._routing = None  # dictionary of routing connections
         self._paths = None  # routing sequence from each segment to outlet
 
+        # dictionary of elevations at the upstream ends of flowlines
+        self.elevup = dict(zip(self.df.id, self.df.elevup))
+        # static dictionary of original flowline routing
+        self._original_routing = self.routing.copy()
+
+
     @property
     def attr_to_m(self):
         if self.attr_length_units == 'feet':
@@ -293,7 +299,7 @@ class Lines:
             prjfile = shapefile.replace('.shp', '.prj')
             prjfile = prjfile if os.path.exists(prjfile) else None
 
-        shpfile_crs = crs(epsg=epsg, proj_str=proj_str, prjfile=prjfile)
+        shpfile_crs = CRS(epsg=epsg, proj_str=proj_str, prjfile=prjfile)
 
         # ensure that filter bbox is in same crs as flowlines
         if filter is not None and not isinstance(filter, tuple):
@@ -425,7 +431,7 @@ class Lines:
                                   epsg=epsg, proj_str=proj_str, prjfile=prjfile)
 
 
-    def to_sfr(self, grid=None, sr=None,
+    def to_sfr(self, grid=None,
                active_area=None, isfr=None,
                model=None,
                model_length_units='undefined',
@@ -433,7 +439,7 @@ class Lines:
                minimum_reach_width=1.,
                cull_flowlines_to_active_area=True,
                consolidate_conductance=False, one_reach_per_cell=False,
-               model_name=None,
+               package_name=None,
                **kwargs):
         """Create a streamflow routing dataset from the information
         in sfrmaker.lines class instance and a supplied sfrmaker.grid class instance.
@@ -459,7 +465,7 @@ class Lines:
             If True, streambed conductance in each reach is consolidated
             (consolidate_conductance = True), and additional reaches besides
             the most downstream reach are dropped.
-        model_name : str
+        package_name : str
             Base name for writing sfr output.
         kwargs : keyword arguments to sfrmaker.sfrdata
 
@@ -471,16 +477,18 @@ class Lines:
         print("\nCreating sfr dataset...")
         totim = time.time()
 
-        if grid is None and sr is not None:
-            print('\nCreating grid class instance from flopy SpatialReference...')
-            ta = time.time()
-            grid = StructuredGrid.from_sr(sr, active_area=active_area, isfr=isfr)
-            print("grid class created in {:.2f}s\n".format(time.time() - ta))
-        elif isinstance(grid, flopy.discretization.StructuredGrid):
+        if flopy and active_area is None and isfr is None and model is not None:
+            if model.version == 'mf6':
+                isfr = np.sum(model.dis.idomain.array == 1, axis=0) > 0
+            else:
+                isfr = np.sum(model.bas6.ibound.array == 1, axis=0) > 0
+        if flopy and isinstance(grid, flopy.discretization.StructuredGrid):
             print('\nCreating grid class instance from flopy Grid instance...')
             ta = time.time()
             grid = StructuredGrid.from_modelgrid(grid, active_area=active_area, isfr=isfr)
             print("grid class created in {:.2f}s\n".format(time.time() - ta))
+        elif flopy and model is not None:
+            grid = StructuredGrid.from_modelgrid(model.modelgrid, active_area=active_area, isfr=isfr)
         elif not isinstance(grid, sfrmaker.grid.Grid):
             raise TypeError('Unrecognized input for grid: {}'.format(grid))
 
@@ -503,12 +511,14 @@ class Lines:
                 self.cull(grid.active_area, inplace=True, simplify=True, tol=2000)
             elif grid._bounds is not None:  # cull to grid bounding box if already computed
                 self.cull(box(*grid._bounds), inplace=True)
-        if model_name is None:
-            model_name = 'model'
+        if package_name is None:
+            if model is not None:
+                package_name = model.name
+            else:
+                package_name = 'model'
 
         # convert routing connections (toid column) from lists (one-to-many)
         # to ints (one-to-one or many-to-one)
-        self.elevup = dict(zip(self.df.id, self.df.elevup))
         routing = self.routing.copy()
 
         # one to many routing is not supported
@@ -700,6 +710,6 @@ class Lines:
         rd = rd[[c for c in SFRData.rdcols if c in rd.columns]].copy()
         sfrd = SFRData(reach_data=rd, segment_data=sd, grid=grid,
                        model=model, model_length_units=model_length_units,
-                       model_name=model_name, **kwargs)
+                       package_name=package_name, **kwargs)
         print("\nTime to create sfr dataset: {:.2f}s\n".format(time.time() - totim))
         return sfrd
