@@ -1,6 +1,7 @@
 import os
 import time
 from packaging import version
+import yaml
 import numpy as np
 import pandas as pd
 import rasterio
@@ -11,14 +12,16 @@ from sfrmaker.routing import find_path, renumber_segments
 from .checks import valid_rnos, valid_nsegs, rno_nseg_routing_consistent
 from .elevations import smooth_elevations
 from .flows import add_to_perioddata, add_to_segment_data
-from .gis import export_reach_data, project, crs
+from .gis import export_reach_data, project, CRS
 from .observations import write_gage_package, write_mf6_sfr_obsfile, add_observations
 from .units import convert_length_units, itmuni_values, lenuni_values
-from .utils import get_sfr_package_format
+from .utils import get_sfr_package_format, get_input_arguments, assign_layers, update
 import sfrmaker
 from sfrmaker.base import DataPackage
 from sfrmaker.mf5to6 import segment_data_to_period_data
+from sfrmaker.reaches import consolidate_reach_conductances
 from sfrmaker.rivdata import RivData
+
 try:
     import flopy
     fm = flopy.modflow
@@ -120,17 +123,20 @@ class SFRData(DataPackage):
                 'roughch': 0.037,
                 'strthick': 1,
                 'strhc1': 1,
+                'istcb2': 223,
+                'gage_starting_unit_number': 228
                 }
+    package_type = 'sfr'
 
     def __init__(self, reach_data=None,
-                 segment_data=None, grid=None, sr=None,
+                 segment_data=None, grid=None,
                  model=None,
                  isfr=None,
                  model_length_units="undefined", model_time_units='d',
                  enforce_increasing_nsegs=True,
-                 package_name=None,
+                 package_name='model',
                  **kwargs):
-        super().__init__(grid=grid, sr=sr, model=model, isfr=isfr,
+        DataPackage.__init__(self, grid=grid, model=model, isfr=isfr,
                          model_length_units=model_length_units,
                          model_time_units=model_time_units,
                          package_name=package_name)
@@ -171,6 +177,10 @@ class SFRData(DataPackage):
         self.model = model  # attached flopy model instance
         # self._ModflowSfr2 = None # attached instance of flopy modflow_sfr2 package object
 
+        # MODFLOW-2005 gages will be assigned sequential unit numbers
+        # starting at gage_starting_unit_number
+        self.gage_starting_unit_number = self.defaults['gage_starting_unit_number']
+
     @property
     def const(self):
         const = self.len_const[self._lenuni] * \
@@ -330,7 +340,12 @@ class SFRData(DataPackage):
         segment_data.sort_values(by=['per', 'nseg'], inplace=True)
         segment_data.index = range(len(segment_data))
         for c in segment_data.columns:
-            sd[c] = segment_data[c].astype(SFRData.dtypes.get(c, np.float32))
+            values = segment_data[c].astype(SFRData.dtypes.get(c, np.float32))
+            # fill any nan values with 0 (same as empty segment_data;
+            # for example elevation if it wasn't specified and
+            # will be sampled from the DEM)
+            values.fillna(0, inplace=True)
+            sd[c] = values
 
         # assign defaults to segment data
         for k, v in self.defaults.items():
@@ -377,29 +392,29 @@ class SFRData(DataPackage):
 
     def add_to_perioddata(self, data, flowline_routing=None,
                           variable='inflow',
-                          line_id_column_in_data=None,
-                          rno_column_in_data=None,
-                          period_column_in_data='per',
-                          variable_column_in_data='Q_avg'):
+                          line_id_column=None,
+                          rno_column=None,
+                          period_column='per',
+                          data_column='Q_avg'):
         return add_to_perioddata(self, data, flowline_routing=flowline_routing,
                                  variable=variable,
-                                 line_id_column_in_data=line_id_column_in_data,
-                                 rno_column_in_data=rno_column_in_data,
-                                 period_column_in_data=period_column_in_data,
-                                 variable_column_in_data=variable_column_in_data)
+                                 line_id_column=line_id_column,
+                                 rno_column=rno_column,
+                                 period_column=period_column,
+                                 data_column=data_column)
 
     def add_to_segment_data(self, data, flowline_routing,
                             variable='flow',
-                            line_id_column_in_data=None,
-                            segment_column_in_data='segment',
-                            period_column_in_data='per',
-                            variable_column_in_data='Q_avg'):
+                            line_id_column=None,
+                            segment_column='segment',
+                            period_column='per',
+                            data_column='Q_avg'):
         return add_to_segment_data(self, data, flowline_routing,
                                    variable=variable,
-                                   line_id_column_in_data=line_id_column_in_data,
-                                   segment_column_in_data=segment_column_in_data,
-                                   period_column_in_data=period_column_in_data,
-                                   variable_column_in_data=variable_column_in_data)
+                                   line_id_column=line_id_column,
+                                   segment_column=segment_column,
+                                   period_column=period_column,
+                                   data_column=data_column)
     @property
     def paths(self):
         """Dict listing routing sequence for each segment
@@ -545,6 +560,36 @@ class SFRData(DataPackage):
                            sd0.outseg,
                            increasing=increasing)
 
+    def assign_layers(self, adjusted_botm_output_path='.'):
+        """Assign model layers to SFR reaches, using the discretzation
+        package in the attached model. New botm elevations for the model
+        will be written to a text array file, if any streambed bottoms
+        are below the model bottom.
+
+        Parameters
+        ----------
+        adjusted_botm_output_path : str
+            Path for writing the text array of adjusted model bottom
+            elevations, by default, '.'
+        """
+        if self.model is not None and hasattr(self.model, 'dis'):
+            botm = self.model.dis.botm.array.copy()
+            nlay = botm.shape[0] + 1
+            layers, new_botm = assign_layers(self.reach_data, botm_array=botm)
+            self.reach_data['k'] = layers
+            if new_botm is not None:
+                outfile = '{}_layer_{}_new_botm_elevations.dat'.format(self.package_name,
+                                                                       nlay)
+                outfile = os.path.join(adjusted_botm_output_path, outfile)
+                np.savetxt(outfile, new_botm, fmt='%.2f')
+                msg = ('Sfrmaker pushed some model botm elevations downward'
+                       'to accomodate streambed bottoms. New botm elevations'
+                       'for layer {} written to {}'.format(nlay, outfile))
+                print(msg)
+        else:
+            print('Need a model instance with a discretization package to assign layers. '
+                  'A model can be assigned to the SFRData.model attribute.')
+
     def create_modflow_sfr2(self, model=None, const=None,
                             isfropt=1,  # override flopy default of 0
                             unit_number=None,
@@ -569,7 +614,8 @@ class SFRData(DataPackage):
         # to fill in required "None" values when writing mf6 input
         elif model is None or model.version == 'mf6':
             model_ws = '.' if model is None else model.model_ws
-            m = fm.Modflow(model_ws=model_ws, structured=self.structured)
+            m = fm.Modflow(modelname=self.package_name, model_ws=model_ws,
+                           structured=self.structured)
             if model is not None:
                 nper = 1
                 if 'tdis' in model.simulation.package_key_dict.keys():
@@ -596,6 +642,7 @@ class SFRData(DataPackage):
             self.segment_data['width2'] = [width2[s] for s in self.segment_data.nseg]
 
         assert not np.any(np.isnan(self.segment_data))
+        
         # create record array for each stress period
         sd = self.segment_data.groupby('per')
         sd = {per: sd.get_group(per).drop('per', axis=1).to_records(index=False)
@@ -654,10 +701,10 @@ class SFRData(DataPackage):
                                          sim_ws='')
             m = flopy.mf6.ModflowGwf(sim)
 
-        # create an sfrmaker.mf6sfr instance
-        from .mf5to6 import mf6sfr
+        # create an sfrmaker.Mf6SFR instance
+        from .mf5to6 import Mf6SFR
 
-        sfr6 = mf6sfr(self.modflow_sfr2)
+        sfr6 = Mf6SFR(self.modflow_sfr2)
 
         # package data
         # An error occurred when storing data "packagedata" in a recarray.
@@ -838,9 +885,9 @@ class SFRData(DataPackage):
 
         # get the CRS and pixel size for the DEM
         with rasterio.open(dem) as src:
-            proj_str = src.crs.to_string()
+            wkt = src.crs.wkt
             epsg = src.crs.to_epsg()
-            raster_crs = crs(epsg=epsg, proj_str=proj_str)
+            raster_crs = CRS(epsg=epsg, wkt=wkt)
 
             # make sure buffer is large enough for DEM pixel size
             buffer_distance = np.max([np.sqrt(src.res[0] *
@@ -980,7 +1027,7 @@ class SFRData(DataPackage):
         if version is None:
             version = get_sfr_package_format(sfrpackagefile)
         if package_name is None:
-            package_name, _ = os.splitext(sfrpackagefile)
+            package_name, _ = os.path.splitext(sfrpackagefile)
         # load the model and SFR package
         if namefile is not None:
             model_ws, namefile = os.path.split(namefile)
@@ -1015,14 +1062,245 @@ class SFRData(DataPackage):
 
     @classmethod
     def from_tables(cls, reach_data, segment_data,
-                    grid=None, sr=None, isfr=None):
+                    grid=None, isfr=None):
         reach_data = pd.read_csv(reach_data)
         segment_data = pd.read_csv(segment_data)
         return cls(reach_data=reach_data, segment_data=segment_data,
-                   grid=grid, sr=sr,
-                   isfr=isfr)
+                   grid=grid, isfr=isfr)
 
-    def to_riv(self, segments=None, rno=None, drop_in_sfr=True):
+    @classmethod
+    def from_yaml(cls, config_file, package_name=None, output_path=None,
+                  write_output=True):
+        """Create an SFRData instance from a yaml-format configuration file.
+
+        Parameters
+        ----------
+        config_file : str or mapping
+            Path to Sfrmaker configuration file in yaml format,
+            or a dictionary-like mapping.
+        output_path : str
+            Where output files will be saved. Default is '.'
+            (in default_config.yml).
+        package_name : str
+            Optional argument for naming the output SFR package; otherwise
+            can be specified in configuration file.
+        write_output : bool
+            Whether or not to write output files, including package file,
+            reach and segment data tables and shapefiles.
+
+        Returns
+        -------
+        sfrdata : sfrmaker.SFRData instance
+        """
+        # load the configuration file
+        wd = os.getcwd()
+        if isinstance(config_file, str):
+            # change the cwd to the config file path
+            path, config_file = os.path.split(config_file)
+            path = path if len(path) > 0 else '.'
+            os.chdir(path)
+            with open(config_file) as src:
+                cfg = yaml.load(src, Loader=yaml.Loader)
+        # or accept a mapping as input
+        else:
+            cfg = config_file
+
+        # read in the default configuration
+        defaults_file = os.path.join(os.path.split(__file__)[0],
+                                     'default_config.yml')
+        with open(defaults_file) as src:
+            defaults = yaml.load(src, Loader=yaml.Loader)
+
+        # add defaults to configuration
+        cfg = update(defaults, cfg)
+
+        # set the package_name and output paths
+        if package_name is not None:
+            _, package_name = os.path.split(package_name)
+            package_name, _ = os.path.splitext(package_name)
+        elif 'package_name' in cfg:
+            _, package_name = os.path.split(cfg['package_name'])
+            package_name, _ = os.path.splitext(package_name)
+        elif 'simulation' in cfg:
+            package_name, _ = os.path.splitext(cfg['simulation']['sim_name'])
+        elif 'model' in cfg:
+            package_name, _ = os.path.splitext(cfg['model']['namefile'])
+        if output_path is None:
+            output_path = cfg['output_path']
+        if not os.path.isdir(output_path):
+            os.makedirs(output_path)
+
+        # model grids from shapefiles or flopy grids are supported
+        grid = None
+        if 'modelgrid' in cfg:
+            grid_cfg = cfg['modelgrid']
+            if 'shapefile' in grid_cfg:
+                grid_kwargs = get_input_arguments(grid_cfg, sfrmaker.StructuredGrid.from_shapefile)
+                grid = sfrmaker.StructuredGrid.from_shapefile(**grid_kwargs)
+            elif not flopy:
+                raise ImportError("Specifying a modelgrid with xoffset, yoffset, etc. requires flopy.")
+            else:
+                # convert delr/delc to ndarrays
+                for delrc, ncells in {'delr': 'ncol', 'delc': 'nrow'}.items():
+                    if np.isscalar(grid_cfg[delrc]):
+                        if ncells not in grid_cfg:
+                            msg = ("Scalar value for {} requires "
+                                   "specification of {}.".format(delrc, ncells))
+                            raise KeyError(msg)
+                        grid_cfg[delrc] = np.array([grid_cfg[delrc]] * grid_cfg[ncells])
+                grid_cfg['xoff'] = grid_cfg.get('xoffset', grid_cfg.get('xoff', 0.))
+                grid_cfg['yoff'] = grid_cfg.get('yoffset', grid_cfg.get('yoff', 0.))
+                grid_kwargs = get_input_arguments(grid_cfg, flopy.discretization.StructuredGrid)
+                grid = flopy.discretization.StructuredGrid(**grid_kwargs)
+
+        # create a Lines instance
+        no_lines_msg = "A flowlines: block must be specified in configuration file."
+        if 'flowlines' not in cfg:
+            raise KeyError(no_lines_msg)
+        lines_config = cfg['flowlines']
+        # custom hydrography option
+        if 'filename' in lines_config:
+            lines_config['shapefile'] = lines_config['filename']
+            lines_kwargs = get_input_arguments(lines_config, sfrmaker.Lines.from_shapefile)
+            lines = sfrmaker.Lines.from_shapefile(**lines_kwargs)
+        # nhdplus option
+        else:
+            # renames to deal with case issues
+            renames = {'nhdplus_paths': 'NHDPlus_paths',
+                       'nhdflowlines': 'NHDFlowlines',
+                       'plusflowlinevaa': 'PlusFlowlineVAA',
+                       'plusflow': 'PlusFlow'}
+            lines_config = {renames.get(k, k): v for k, v in lines_config.items()}
+            lines_kwargs = get_input_arguments(lines_config, sfrmaker.Lines.from_nhdplus_v2)
+            lines = sfrmaker.Lines.from_nhdplus_v2(**lines_kwargs)
+
+        # load a model if there is one
+        # modflow 6
+        model_version = cfg.get('package_version', 'mf6')
+        no_flopy_msg = "Specifying a model requires flopy."
+        if 'simulation' in cfg:
+            if not flopy:
+                raise ImportError(no_flopy_msg)
+            sim_kwargs = get_input_arguments(cfg['simulation'], flopy.mf6.MFSimulation)
+            sim = mf6.MFSimulation.load(**sim_kwargs)
+            model = sim.get_model(model_name=cfg['model']['modelname'])
+            model_version = model.version
+            if grid is None:
+                grid = model.modelgrid
+        # modflow-2005
+        elif 'model' in cfg:
+            if not flopy:
+                raise ImportError(no_flopy_msg)
+            cfg['model']['f'] = cfg['model']['namefile']
+            model_kwargs = get_input_arguments(cfg['model'], flopy.modflow.Modflow.load)
+            model = flopy.modflow.Modflow.load(**model_kwargs)
+            model_version = model.version
+            if grid is None:
+                grid = model.modelgrid
+        else:
+            model = None
+
+        # create an SFRData instance
+        to_sfr_kwargs = get_input_arguments(cfg, sfrmaker.Lines.to_sfr)
+        to_sfr_kwargs['active_area'] = cfg.get('active_area', {}).get('filename')
+        to_sfr_kwargs['model'] = model
+        to_sfr_kwargs['grid'] = grid
+        to_sfr_kwargs['package_name'] = package_name
+        # to_sfr() populates isfr from model if neither active_area or isfr are argued
+        sfrdata = lines.to_sfr(**to_sfr_kwargs)
+        sfrdata._tables_path = os.path.join(output_path, sfrdata._tables_path)
+        sfrdata._shapefiles_path = os.path.join(output_path, sfrdata._shapefiles_path)
+
+        # setup elevations
+        if cfg.get('set_streambed_top_elevations_from_dem', False):
+            error_msg = ("set_streambed_top_elevations_from_dem=True "
+                         "requires a dem: block.")
+            if 'dem' not in cfg:
+                raise KeyError(error_msg)
+            elevation_units = cfg['dem'].get('elevation_units')
+            sfrdata.set_streambed_top_elevations_from_dem(cfg['dem']['filename'],
+                                                          dem_z_units=elevation_units)
+        else:
+            sfrdata.reach_data['strtop'] = sfrdata.interpolate_to_reaches('elevup', 'elevdn')
+
+        # assign layers to the sfr reaches
+        if model is not None:
+            sfrdata.assign_layers(adjusted_botm_output_path=output_path)
+
+        # option to convert reaches to the River Package
+        if 'to_riv' in cfg:
+            rivdata = sfrdata.to_riv(line_ids=cfg['to_riv'],
+                                     drop_in_sfr=True)
+            sfrdata.setup_riv(rivdata)
+            rivdata.write_table()
+            rivdata.write_shapefiles()
+            print('to_riv option: output table and shapefiles written, but'
+                  'writing of RIV package not implemented yet. Use flopy to'
+                  'write the RIV package.')
+
+        # add inflows
+        if 'inflows' in cfg:
+            inflows_input = cfg['inflows']
+            # resample inflows to model stress periods
+            inflows_input['id_column'] = inflows_input['line_id_column']
+            inflows_by_stress_period = pd.read_csv(inflows_input['filename'])
+
+            # check if all inflow sites are included in sfr network
+            missing_sites = set(inflows_by_stress_period[inflows_input['id_column']]). \
+                difference(lines._original_routing.keys())
+            if any(missing_sites):
+                inflows_routing_input = cfg.get('inflows_routing')
+                if inflows_routing_input is None:
+                    raise KeyError(('inflow sites {} are not within the model sfr network. '
+                                    'Please supply an inflows_routing: block'.format(missing_sites)))
+                routing = pd.read_csv(inflows_routing_input['filename'])
+                routing = dict(zip(routing[inflows_routing_input['id_column']],
+                                   routing[inflows_routing_input['routing_column']]))
+            else:
+                routing = lines._original_routing
+            missing_sites = any(set(inflows_by_stress_period[inflows_input['id_column']]). \
+                                difference(routing.keys())),
+            if any(missing_sites):
+                raise KeyError(('Inflow sites {} not found in {}'.format(missing_sites,
+                                                                         inflows_routing_input['filename'])))
+
+            # add resampled inflows to SFR package
+            inflows_input['data'] = inflows_by_stress_period
+            inflows_input['flowline_routing'] = routing
+            if model_version == 'mf6':
+                inflows_input['variable'] = 'inflow'
+                method = sfrdata.add_to_perioddata
+            else:
+                method = sfrdata.add_to_segment_data
+            kwargs = get_input_arguments(inflows_input.copy(), method)
+            method(**kwargs)
+
+        # add observations
+        if 'observations' in cfg:
+            if not model:
+                pass #  raise KeyError('Setup of observations input results a model: block')
+            if model_version != 'mf6':
+                sfrdata.gage_starting_unit_number = cfg['gag']['starting_unit_number']
+            key = 'filename' if 'filename' in cfg['observations'] else 'filenames'
+            cfg['observations']['data'] = cfg['observations'][key]
+            kwargs = get_input_arguments(cfg['observations'].copy(), sfrdata.add_observations)
+            obsdata = sfrdata.add_observations(**kwargs)
+
+        if write_output:
+            if model is None:
+                package_file_path = os.path.join(output_path, package_name + '.sfr')
+            else:
+                package_file_path = os.path.join(model.model_ws, package_name + '.sfr')
+            sfrdata.write_package(package_file_path,
+                                  version=model_version)
+            sfrdata.write_tables()
+            sfrdata.write_shapefiles()
+
+        # change the cwd back
+        os.chdir(wd)
+        return sfrdata
+
+    def to_riv(self, segments=None, rno=None, line_ids=None, drop_in_sfr=True):
         """Cast one or more reaches to a RivData instance,
         which can then be written as input to the MODFLOW RIV package.
 
@@ -1031,9 +1309,14 @@ class SFRData(DataPackage):
         segments : sequence of ints
             Convert the listed segments, and all downstream segments,
             to the RIV package. If None, all segments are converted (default).
-        nro : sequence of ints
+        rno : sequence of ints
             Convert the listed reach numbers (nro), and all downstream reaches,
             to the RIV package. If None, all reaches are converted (default).
+        line_ids : sequence of ints
+            Convert the segments corresponding to the line_ids 
+            (unique identifiers for LineString features in the source hydrography), 
+            and all downstream segments to the RIV package. If None, all reaches 
+            are converted (default).
         drop_in_sfr : bool
             Whether or not to remove the converted segments from the SFR package
             (default True)
@@ -1044,20 +1327,24 @@ class SFRData(DataPackage):
         """
 
         # get the downstream segments to be converted
-        if segments is None and rno is None:
-            loc = slice(None, None)  # all reaches
+        loc = np.array([True] * len(self.reach_data))
+        if segments is None and rno is None and line_ids is None:
+            #loc = slice(None, None)  # all reaches
             # if all reaches are being converted,
             # skip dropping the reaches from the sfr package
             drop_in_sfr = False
-        elif segments is not None:
+        if segments is not None:
             if np.isscalar(segments):
                 segments = [segments]
-            loc = self.reach_data.iseg.isin(segments)
-        elif rno is not None:
+            loc = loc & self.reach_data.iseg.isin(segments)
+        if line_ids is not None:
+            if np.isscalar(line_ids):
+                line_ids = [line_ids]
+            loc = loc & self.reach_data.line_id.isin(line_ids)
+        if rno is not None:
             if np.isscalar(rno):
                 rno = [rno]
-            loc = self.reach_data.rno.isin(rno)
-
+            loc = loc & self.reach_data.rno.isin(rno)
         reaches = self.reach_data.loc[loc, 'rno'].tolist()
         to_riv_reaches = set()
         for rno in reaches:
@@ -1070,12 +1357,15 @@ class SFRData(DataPackage):
         is_riv_reach = self.reach_data.rno.isin(to_riv_reaches)
         df = self.reach_data.loc[is_riv_reach].copy()
 
+        # consolidate the reaches to 1 per cell
+        df = consolidate_reach_conductances(df, keep_only_dominant=True)
+
         # add in a column for the stress period
         # even though transfer of any specified stage changes
         # aren't implemented yet
         df['per'] = 0
 
-        df['cond'] = df.width * df.rchlen * df.strhc1 / df.strthick
+        df['cond'] = df['Cond_sum']
 
         if any(self.segment_data.depth1 > 0):
             # find period with most data
@@ -1130,16 +1420,43 @@ class SFRData(DataPackage):
             self._reset_routing()
         return riv
 
+    def run_diagnostics(self, checkfile=None, **kwargs):
+        """Run the Flopy SFR diagnostic suite.
+
+        Parameters
+        ----------
+        checkfile : str
+            Path of file to write results to, by default, results
+            are written to {}_SFR.chk, where {} is the
+            SFRData.package_name attribute.
+        kwargs : keyword arguments to flopy.modflow.ModflowSfr2.check()
+
+        """
+        if flopy:
+            print('\nRunning Flopy v. {} diagnostics...'.format(flopy.__version__))
+            self.create_modflow_sfr2(model=self.model)
+            if checkfile is None:
+                checkfile = '{}_SFR.chk'.format(self.package_name)
+            self._ModflowSfr2.check(checkfile, **kwargs)
+        print('wrote {}'.format(checkfile))
+
     def write_package(self, filename=None, version='mf2005', idomain=None,
-                      options=None, write_observations_input=True,
+                      options=None, run_diagnostics=True,
+                      write_observations_input=True,
+                      external_files_path=None, gage_starting_unit_number=None,
                       **kwargs):
-        """Write and SFR package file.
+        """Write SFR package input.
 
         Parameters
         ----------
         version : str
             'mf2005' or 'mf6'
         """
+        print('SFRmaker v. {}'.format(sfrmaker.__version__))
+        # run the flopy SFR diagnostics
+        if run_diagnostics:
+            self.run_diagnostics()
+
         # recreate the flopy package object in case it changed
         self.create_modflow_sfr2(model=self.model, **kwargs)
         header_txt = "#SFR package created by SFRmaker v. {}, " \
@@ -1150,18 +1467,22 @@ class SFRData(DataPackage):
 
         if filename is None:
             filename = self.modflow_sfr2.fn_path
-        if version == 'mf2005':
+        if gage_starting_unit_number is None:
+            gage_starting_unit_number = self.gage_starting_unit_number
+        if version in {'mf2005', 'mfnwt'}:
 
             if write_observations_input and len(self.observations) > 0:
                 gage_package_filename = os.path.splitext(filename)[0] + '.gage'
-                self.write_gage_package(filename=gage_package_filename)
+                self.write_gage_package(filename=gage_package_filename,
+                                        gage_starting_unit_number=gage_starting_unit_number)
 
             self.modflow_sfr2.write_file(filename=filename)
+            print('wrote {}.'.format(filename))
 
         elif version == 'mf6':
 
-            # instantiate mf6sfr converter object with mf-nwt model/sfr package from flopy
-            from .mf5to6 import mf6sfr
+            # instantiate Mf6SFR converter object with mf-nwt model/sfr package from flopy
+            from .mf5to6 import Mf6SFR
             if options is None:
                 # save budget and stage output by default
                 options = ['save_flows',
@@ -1173,34 +1494,53 @@ class SFRData(DataPackage):
                 self.write_mf6_sfr_obsfile(filename=obs_input_filename)
                 options.append('OBS6 FILEIN {}'.format(obs_input_filename))
 
-            sfr6 = mf6sfr(self.modflow_sfr2, period_data=self.period_data,
+            sfr6 = Mf6SFR(SFRData=self, period_data=self.period_data,
                           idomain=idomain,
                           options=options)
 
             # write a MODFLOW 6 file
-            sfr6.write_file(filename=filename)
+            sfr6.write_file(filename=filename, external_files_path=external_files_path)
 
-    def write_tables(self, filepath='./sfr'):
+    def write_tables(self, basename=None):
+        if basename is None:
+            output_path = self._tables_path
+            if not os.path.isdir(output_path):
+                os.makedirs(output_path)
+            basename = self.package_name
+        else:
+            output_path, basename = os.path.split(basename)
+            basename, _ = os.path.splitext(basename)
+        reach_data_file = os.path.normpath('{}/{}_sfr_reach_data.csv'.format(output_path, basename))
+        self.reach_data.drop('geometry', axis=1).to_csv(reach_data_file, index=False)
+        print('wrote {}'.format(reach_data_file))
+        segment_data_file = os.path.normpath('{}/{}_sfr_segment_data.csv'.format(output_path, basename))
+        self.segment_data.to_csv(segment_data_file, index=False)
+        print('wrote {}'.format(segment_data_file))
 
-        filepath, file_extension = os.path.splitext(filepath)
-        self.reach_data.drop('geometry', axis=1).to_csv('{}_reach_data.csv'.format(filepath), index=False)
-        self.segment_data.to_csv('{}_segment_data.csv'.format(filepath), index=False)
+        if self.period_data is not None and len(self.period_data) > 0:
+            pd_file = os.path.normpath('{}/{}_sfr_period_data.csv'.format(output_path, basename))
+            self.period_data.dropna(axis=1, how='all').to_csv(pd_file, index=False)
+            print('wrote {}'.format(pd_file))
 
-    def write_gage_package(self,
-                           filename=None,
-                           gage_package_unit=25,
-                           start_gage_unit=228):
+    def write_gage_package(self, filename=None, gage_package_unit=25,
+                           gage_starting_unit_number=None):
         if filename is None:
             filename = self.observations_file
         else:
             self.observations_file = filename
+        if gage_starting_unit_number is None:
+            gage_starting_unit_number = self.gage_starting_unit_number
         gage_namfile_entries_file = filename + '.namefile_entries'
+        model = self.model
+        if model is None:
+            model = self.modflow_sfr2.parent
+            model.model_ws = os.path.split(filename)[0]
         return write_gage_package(self.observations,
                                   gage_package_filename=filename,
                                   gage_namfile_entries_file=gage_namfile_entries_file,
-                                  model=self.model,
+                                  model=model,
                                   gage_package_unit=gage_package_unit,
-                                  start_gage_unit=start_gage_unit)
+                                  start_gage_unit=gage_starting_unit_number)
 
     def write_mf6_sfr_obsfile(self,
                               filename=None,
@@ -1215,13 +1555,6 @@ class SFRData(DataPackage):
                                      filename,
                                      sfr_output_filename)
 
-    def export_cells(self, filename=None, nodes=None, geomtype='polygon'):
-        """Export shapefile of model cells with stream reaches."""
-        if filename is None:
-            filename = self.package_name + '_sfrcells.shp'
-        export_reach_data(self.reach_data, self.grid, filename,
-                          nodes=nodes, geomtype=geomtype)
-
     def export_outlets(self, filename=None):
         """Export shapefile of model cells with stream reaches."""
         if filename is None:
@@ -1230,17 +1563,6 @@ class SFRData(DataPackage):
         export_reach_data(self.reach_data, self.grid, filename,
                           nodes=nodes, geomtype='point')
 
-    def export_lines(self, filename=None):
-        """Export shapefile of linework"""
-        if filename is None:
-            filename = self.package_name + '_sfrlines.shp'
-        rd = self.reach_data
-        assert 'geometry' in rd.columns and \
-               isinstance(rd.geometry.values[0], LineString), \
-            "No LineStrings in reach_data.geometry"
-        df2shp(rd, filename, epsg=self.grid.crs.epsg,
-               proj_str=self.grid.crs.proj_str)
-
     def export_routing(self, filename=None):
         """Export linework shapefile showing all routing connections between SFR reaches.
         A length field containing the distance between connected reaches
@@ -1320,43 +1642,15 @@ class SFRData(DataPackage):
 
         export_reach_data(rd, self.grid, filename, geomtype='point')
 
-    def export_period_data(self, filename=None, geomtype='point'):
-        """Export point shapefile showing locations of period data
-        in a MODFLOW-6 SFR package (e.g. inflows, runoff, etc.)
-
-        Parameters
-        ----------
-        f : str, filename
-        geomtype : str ('point' or 'polygon')
-            write the locations as points at the cell centers, or polygons
-            of the model cells containing the period data.
-
-        """
-        data = self.period_data.dropna(axis=1).sort_values(by=['per', 'rno'])
-        nodes = dict(zip(self.reach_data.rno, self.reach_data.node))
-
-        for var in ['evaporation', 'inflow', 'rainfall', 'runoff', 'stage']:
-            if var in data.columns:
-                # pivot the segment data to segments x periods with values of varname
-                aggfunc = 'mean'  # how to aggregate multiple instances of rno/per combinations
-                if var in ['inflow', 'runoff']:
-                    aggfunc = 'sum'
-                df = data.reset_index(drop=True).pivot_table(index='rno', columns='per', values=var,
-                                                             aggfunc=aggfunc).reset_index()
-                # rename the columns to indicate stress periods
-                df.columns = ['rno'] + ['{}{}'.format(i, var) for i in range(df.shape[1] - 1)]
-                df['node'] = [nodes[rno] for rno in df['rno']]
-                if filename is None:
-                    filename = self.package_name + '_sfr_period_data_{}.shp'.format(var)
-                elif var not in filename:
-                    filename = os.path.splitext(filename)[0] + '_{}.shp'.format(var)
-                export_reach_data(df, self.grid, filename, geomtype=geomtype)
-
     def export_observations(self, filename=None, geomtype='point'):
 
         data = self.observations
+        if len(data) == 0:
+            print('No observations to export!')
+            return
         nodes = dict(zip(self.reach_data.rno, self.reach_data.node))
         data['node'] = [nodes[rno] for rno in data['rno']]
         if filename is None:
             filename = self.observations_file + '.shp'
-        export_reach_data(data, self.grid, filename, geomtype=geomtype)
\ No newline at end of file
+        export_reach_data(data, self.grid, filename, geomtype=geomtype)
+
