@@ -3,8 +3,9 @@
 import json
 
 import numpy as np
+from flopy.discretization import StructuredGrid
 from GISio import shp2df, df2shp
-from flopy.utils.reference import SpatialReference
+
 
 # basic grid parameters
 name = 'map_test'
@@ -12,6 +13,7 @@ epsg = 5070
 xul, yul = 520487.3, 1194668.3
 nrow, ncol = 20, 20
 dxy = 5280 * .3048
+xoff, yoff = xul, yul - nrow * dxy
 buf = 1e4
 bounds = xul - buf, \
          yul - dxy * nrow - buf, \
@@ -24,9 +26,9 @@ df = shp2df('/Users/aleaf/Documents/MAP/repos/sfr_output/preprocessed/flowlines_
 df2shp(df, 'data/{}_flowlines.shp'.format(name), epsg=epsg)
 
 # make a spatial reference object defining the grid
-sr = SpatialReference(delr=np.ones(ncol, dtype=float) * dxy,
+sr = StructuredGrid(delr=np.ones(ncol, dtype=float) * dxy,
                       delc=np.ones(nrow, dtype=float) * dxy,
-                      xul=xul, yul=yul, epsg=epsg)
+                      xoff=xoff, yoff=yoff, epsg=epsg)
 # export sr info to json file
 model_info = sr.attribute_dict
 model_info['nrow'] = sr.nrow
