@@ -155,7 +155,6 @@ def test_create_mf6sfr(mf6sfr, shellmound_sfrdata, shellmound_model):
                     print('values in packagedata.{} != sfrdata.segment_data.{}'.format(col, mf2005col))
 
 
-@pytest.mark.skip('waiting on flopy fix for empty packagedata lines where cellid is None')
 def test_flopy_mf6sfr_outfile(mf6sfr, mf6sfr_outfile):
     assert os.path.exists(mf6sfr_outfile)
     # check that the written package data matches in memory package data
@@ -232,7 +231,8 @@ def test_write_mf6_package(shellmound_sfrdata, mf6sfr, outdir):
     df = pd.read_csv(io.StringIO(text), delim_whitespace=True,
                          #skiprows=2,
                          header=None,
-                         names=cols
+                         names=cols,
+                         index_col=False
                          )
     isna = df.isna().any(axis=1).values.astype(bool)
     df.dropna(axis=0, inplace=True)
@@ -245,6 +245,7 @@ def test_write_mf6_package(shellmound_sfrdata, mf6sfr, outdir):
 
 @pytest.mark.parametrize('kwargs', [{'rno': 1},  # specified reach(es)
                                     {'segments': 1},  # specified segment(s)
+                                    {'line_ids': 17955471},  # specified line numbers in source hydrography
                                     {},  # all segments
                                     ])
 def test_to_riv(shellmound_sfrdata, kwargs):
@@ -267,3 +268,12 @@ def test_to_riv(shellmound_sfrdata, kwargs):
     # would be nice to have a way to compare stages
     # but some sfr were removed that didn't get converted to RIV
     # (minor reaches collocated with reaches that got converted)
+
+
+def test_run_diagnostics(sfrdata):
+    """Check that flopy diagnostics were run
+    (that a .chk output file was produced)."""
+    sfrdata.run_diagnostics()
+    checkfile = os.path.join(sfrdata.model.model_ws,
+                             '{}_SFR.chk'.format(sfrdata.package_name))
+    assert os.path.getsize(checkfile) > 0
